Imports Microsoft.VisualBasic
Imports System
'Imports System.Collections.Generic;
'Imports System.Linq;
'Imports System.Text;
Imports System.IO
' BitArray
Imports System.Collections


Namespace GraphMaxClique
  Friend Class GraphStructureProgram
	Shared Sub Main(ByVal args() As String)
	  Try
		Console.WriteLine(vbLf & "Begin graph for maximum clique demo" & vbLf)

		'string graphFile = "..\\..\\SimpleGraph.txt";
		'MyGraph.ValidateGraphFile(graphFile, "SIMPLE");

		Dim graphFile As String = "..\..\DimacsGraph.clq"
		Console.WriteLine("Validating DIMACS format graph file " & graphFile)
		MyGraph.ValidateGraphFile(graphFile, "DIMACS")
		Console.WriteLine("Graph file validation complete")

		'MyGraph graph = new MyGraph(graphFile, "SIMPLE");
		Console.WriteLine(vbLf & "Loading graph into memory")
		Dim graph As New MyGraph(graphFile, "DIMACS")

		Console.WriteLine(vbLf & "Validating graph data structure")
		graph.ValidateGraph()
		Console.WriteLine("Graph data structure validation complete")

		Console.WriteLine(vbLf & "Graph adjaceny representation: " & vbLf)
		Console.WriteLine(graph.ToString())

                'graph.ValidateGraph()

		Console.WriteLine(vbLf & "Are nodes 5 and 8 adjacent? " & graph.AreAdjacent(5,8))
		Console.WriteLine("Number neighbors of node 4 = " & graph.NumberNeighbors(4))
		'Console.WriteLine(graph.NumberEdges);
		'Console.WriteLine(graph.NumberNeighbors(4));
		'Console.WriteLine(graph.AreAdjacent(2, 4));
		'Console.WriteLine(graph.AreAdjacent(4, 2));
		'Console.WriteLine(graph.AreAdjacent(2, 6));


		Console.WriteLine(vbLf & "End graph for maximum clique demo" & vbLf)
		Console.ReadLine()
	  Catch ex As Exception
		Console.WriteLine("Fatal: " & ex.Message)
		Console.ReadLine()
            End Try
            ' Main
        End Sub
        ' class Program
    End Class

  Public Class MyGraph
	Private data As BitMatrix
	Private _numberNodes As Integer

        Private _numberEdges As Integer
        ' node degrees
        Private _numberNeighbors() As Integer

	Public Sub New(ByVal graphFile As String, ByVal fileFormat As String)
	  If fileFormat.ToUpper() = "DIMACS" Then
		LoadDimacsFormatGraph(graphFile)
	  Else
		Throw New Exception("Format " & fileFormat & " not supported")
	  End If
	End Sub

    ' Private Sub LoadSimpleFormatGraph(ByVal graphFile As String)
	 ' Dim ifs As New FileStream(graphFile, FileMode.Open)
	 ' Dim sr As New StreamReader(ifs)
	 ' Dim line As String = ""
	 ' Dim tokens() As String = Nothing
	 ' Dim subtokens() As String = Nothing
	 ' Dim numNodes As Integer = 0
	 ' Dim numEdges As Integer = 0
	'line = sr.ReadLine()
	'Do While line IsNot Nothing
		'line = line.Trim()
		'If line.StartsWith("//") = False Then
		 ' numNodes += 1
		'End If
		'line = sr.ReadLine()
	'Loop
	  'sr.Close()
	 ' ifs.Close()

	  'Me.data = New BitMatrix(numNodes)
	  'Me.numberNeighbors = New Integer(numNodes - 1){}

	  'ifs = New FileStream(graphFile, FileMode.Open)
	  'sr = New StreamReader(ifs)

	 ' line = sr.ReadLine()
	  'Do While line IsNot Nothing
		'line = line.Trim()
		'If line.StartsWith("//") = True Then
			'line = sr.ReadLine()
			'Continue Do
		'End If
		'tokens = line.Split(":"c)
		'Dim fromNode As Integer = Integer.Parse(tokens(0))
		'subtokens = tokens(1).Split(","c)
		'numEdges += subtokens.Length
		'Me.numberNeighbors(fromNode) = subtokens.Length
		'For j As Integer = 0 To subtokens.Length - 1

		  'Dim toNode As Integer = Integer.Parse(subtokens(j))
		  'Me.data.SetValue(fromNode, toNode, True)
		  'Me.data.SetValue(toNode, fromNode, True)
		'Next j
		  'line = sr.ReadLine()
	  'Loop
	  'sr.Close()
	  'ifs.Close()

	 ' Me.numberNodes = numNodes
	  'Me.numberEdges = numEdges \ 2
	 ' Return
    ' End Sub ' LoadSimpleFormatGraph

	Private Sub LoadDimacsFormatGraph(ByVal graphFile As String)
	  Dim ifs As New FileStream(graphFile, FileMode.Open)
	  Dim sr As New StreamReader(ifs)
	  Dim line As String = ""
	  Dim tokens() As String = Nothing

            ' advance to and get the p line (ex: "p edge 9 16")
            ' read first line of file as a priming read
            line = sr.ReadLine()
            line = line.Trim()
	  Do While line IsNot Nothing AndAlso line.StartsWith("p") = False
		line = sr.ReadLine()
		line = line.Trim()
	  Loop
            tokens = line.Split(" "c)
            ' number nodes
            Dim numNodes As Integer = Integer.Parse(tokens(2))
            ' number edges
            Dim numEdges As Integer = Integer.Parse(tokens(3))

	  sr.Close()
	  ifs.Close()

	  Me.data = New BitMatrix(numNodes)
            ' reopen file
            ifs = New FileStream(graphFile, FileMode.Open)
	  sr = New StreamReader(ifs)
	  line = sr.ReadLine()
	  Do While line IsNot Nothing
                line = line.Trim()
                ' (ex: "e 7 4")
                If line.StartsWith("e") = True Then
                    tokens = line.Split(" "c)
                    ' DIMACS is 1-based. subtract 1 to convert to 0-based
                    Dim nodeA As Integer = Integer.Parse(tokens(1)) - 1
                    Dim nodeB As Integer = Integer.Parse(tokens(2)) - 1
                    data.SetValue(nodeA, nodeB, True)
                    data.SetValue(nodeB, nodeA, True)
                End If
		  line = sr.ReadLine()
	  Loop
	  sr.Close()
	  ifs.Close()

            Me._numberNeighbors = New Integer(numNodes - 1) {}
	  For row As Integer = 0 To numNodes - 1
		Dim count As Integer = 0
		For col As Integer = 0 To numNodes - 1
		  If data.GetValue(row, col) = True Then
			count += 1
		  End If
		Next col
                _numberNeighbors(row) = count
	  Next row

            Me._numberNodes = numNodes
            Me._numberEdges = numEdges
	  Return
	End Sub

	Public ReadOnly Property NumberNodes() As Integer
	  Get
                Return Me._numberNodes
	  End Get
	End Property

	Public ReadOnly Property NumberEdges() As Integer
	  Get
                Return Me._numberEdges
	  End Get
	End Property

	Public Function NumberNeighbors(ByVal node As Integer) As Integer
            Return Me._numberNeighbors(node)
	End Function

	Public Function AreAdjacent(ByVal nodeA As Integer, ByVal nodeB As Integer) As Boolean
	  If Me.data.GetValue(nodeA, nodeB) = True Then
		Return True
	  Else
		Return False
	  End If
	End Function

	Public Overrides Function ToString() As String
	  Dim s As String = ""
            For i = 0 To Me.data.Dim - 1
                s &= i & ": "
                For j = 0 To Me.data.Dim - 1
                    If Me.data.GetValue(i, j) = True Then
                        s &= j & " "
                    End If
                Next j
                s &= Environment.NewLine
            Next i
	  Return s
	End Function

	' ---------------------------------------------------------------------------------------------------------------------------------------------------------

	Public Shared Sub ValidateGraphFile(ByVal graphFile As String, ByVal fileFormat As String)
	  If fileFormat.ToUpper() = "DIMACS" Then
		ValidateDimacsGraphFile(graphFile)
	  Else
		Throw New Exception("Format " & fileFormat & " not supported")
	  End If
	End Sub

       'Public Shared Sub ValidateSimpleGraphFile(ByVal graphFile As String)
	' Dim ifs As New FileStream(graphFile, FileMode.Open)
	' Dim sr As New StreamReader(ifs)
	' Dim line As String = ""
	'  Dim tokens() As String = Nothing
	' Dim subtokens() As String = Nothing

	  'line = sr.ReadLine()
	  'Do While line IsNot Nothing
		'line = line.Trim()
		'If line.StartsWith("//") = True Then
		 ' line = sr.ReadLine()
		'  Continue Do
		'End If
		'Try
		 ' tokens = line.Split(":"c)
		  'Dim fromNode As Integer = Integer.Parse(tokens(0))
		  'subtokens = tokens(1).Split(","c)
		'Catch
		 ' Throw New Exception("Error parsing line = " & line & " in ValidateSimpleGraphFile")
		'End Try
		
             ' line = sr.ReadLine()

	  'Loop 

	  'sr.Close()
	  'ifs.Close()
	  'Return
	'End Sub

	Public Shared Sub ValidateDimacsGraphFile(ByVal graphFile As String)
	  Dim ifs As New FileStream(graphFile, FileMode.Open)
	  Dim sr As New StreamReader(ifs)
	  Dim line As String = ""
	  Dim tokens() As String = Nothing

	  line = sr.ReadLine()
	  Do While line IsNot Nothing
		line = line.Trim()
		If line.StartsWith("c") = False AndAlso line.StartsWith("p") = False AndAlso line.StartsWith("e") = False Then
		  Throw New Exception("Unknown line type: " & line & " in ValidateDimacsGraphFile")
		End If

		Try
		  If line.StartsWith("p") Then
			tokens = line.Split(" "c)
			Dim numNodes As Integer = Integer.Parse(tokens(2))
			Dim numEdges As Integer = Integer.Parse(tokens(3))
		  ElseIf line.StartsWith("e") Then
			tokens = line.Split(" "c)
			Dim nodeA As Integer = Integer.Parse(tokens(1))
			Dim nodeB As Integer = Integer.Parse(tokens(2))
		  End If
		Catch
		  Throw New Exception("Error parsing line = " & line & " in ValidateDimacsGraphFile")
		End Try
		  line = sr.ReadLine()
	  Loop

	  sr.Close()
	  ifs.Close()
            Return
            ' ValidateDimacsGraphFile
        End Sub

	Public Sub ValidateGraph()
	  ' total number edges items must be an even number
	  Dim numConnections As Integer = 0
            For i = 0 To Me.data.Dim - 1
                For j = 0 To Me.data.Dim - 1
                    If Me.data.GetValue(i, j) = True Then
                        numConnections += 1
                    End If
                Next j
            Next i
	  If numConnections Mod 2 <> 0 Then
		Throw New Exception("Total number of connections in graph is " & numConnections & ". Should be even")
	  End If

	  ' fully symmetric
            For i = 0 To Me.data.Dim - 1
                If Me.data.GetValue(i, i) = True Then
                    Throw New Exception("Node " & i & " is connected to itself")
                End If
                For j = 0 To Me.data.Dim - 1
                    If Me.data.GetValue(i, j) <> Me.data.GetValue(j, i) Then
                        Throw New Exception("Graph is not symmetric at " & i & " and " & j)
                    End If
                Next j
            Next i

            Return
            ' ValidateGraph
        End Sub



	' ---------------------------------------------------------------------------------------------------------------------------------------------------------

        Private Class BitMatrix
            ' an array of arrays
            Private data() As BitArray
            ' dimension
            Public ReadOnly [Dim] As Integer

            Public Sub New(ByVal n As Integer)
                Me.data = New BitArray(n - 1) {}
                For i = 0 To data.Length - 1
                    Me.data(i) = New BitArray(n)
                Next i
                Me.Dim = n
            End Sub
            Public Function GetValue(ByVal row As Integer, ByVal col As Integer) As Boolean
                Return data(row)(col)
            End Function
            Public Sub SetValue(ByVal row As Integer, ByVal col As Integer, ByVal value As Boolean)
                data(row)(col) = value
            End Sub
            Public Overrides Function ToString() As String
                Dim s As String = ""
                For i = 0 To data.Length - 1
                    For j = 0 To data(i).Length - 1
                        If data(i)(j) = True Then
                            s &= "1 "
                        Else
                            s &= "0 "
                        End If
                    Next j
                    s &= Environment.NewLine
                Next i
                Return s
            End Function

            ' class BitMatrix
        End Class

        ' class MyGraph
    End Class

  ' ==================================================================================================================================================================

    'Private Class BitMatrix
    'Private data() As BitArray ' an array of arrays
    'Public ReadOnly [Dim] As Integer ' dimension

    'Public Sub New(ByVal n As Integer)
    ' Me.data = New BitArray(n - 1) {}
    'For i  = 0 To data.Length - 1
    ' Me.data(i) = New BitArray(n)
    'Next i
    ' Me.Dim = n
    ''End Sub
    ' Public Function GetValue(ByVal row As Integer, ByVal col As Integer) As Boolean
    'Return data(row)(col)
    'End Function
    'Public Sub SetValue(ByVal row As Integer, ByVal col As Integer, ByVal value As Boolean)
    'data(row)(col) = value
    'End Sub
    ' Public Overrides Function ToString() As String
    'Dim s As String = ""
    ' For i  = 0 To data.Length - 1
    'For j  = 0 To data(i).Length - 1
    'If data(i)(j) = True Then
    's &= "1 "
    'Else
    ' s &= "0 "
    ' End If
    ' Next j
    's &= Environment.NewLine
    ' Next i
    'Return s
    'End Function

    ' class BitMatrix
    ' End Class
    ' ==================================================================================================================================================================


    ' ns
End Namespace